# Data structure readme

## General
- Source text is the text being quoted. Target text is the text quoting the source text.
- The extent of a match between source text and target text is always indicated with a start character position, and an end character position.

## Building blocks
- Each building block has a corresponding Python class in the `passager` package. This file gives a high level description for each class.

### SourceSegment
- One block of text in the source text ranging from `start` to `end`.

### CitationSource
- A group of overlapping `SourceSegments`

### TargetLocation
- One block of text (i.e. a quotation) in the target text ranging from `start` to `end`.

### TargetText
- One scholarly work with a `filename` and a list of `TargetLocations`.

### TargetTextLocationLink
- Links a `TargetLocation` of a `TargetText` to a source segment range from `source_segment_start_id` to `source_segment_end_id`.

### TargetLocationSelection
- A `target_text_id` with a list of `target_location_ids` which is a subset of all `target_location_ids` of the `TargetText` with `target_text_id`.
- Only used in combination with `CitationSourceLink` to map `CitationSources` to `TargetLocations`.

### CitationSourceLink
- Links a `CitationSource` to a `TargetText` and matched `TargetLocations` via `TargetLocationSelections`.

## File documentation

### citation_sources.json
- A list of `CitationSources`. Each `CitationSource` has an ID and a list of `SourceSegments`.
- A `SourceSegment` has an ID, start and end position, a frequency, a length in tokens and the text of the segment.

### citation_source_links.json
- A list of `CitationSourceLinks`.

### target_texts.json
- A list of `TargetTexts`. Each `TargetText` has an ID, a filename and a list of `TargetLocations`.
- A `TargetLocation` has an ID, a start and end position and the text of the location.

### target_text_location_links.json
- A list of `TargetTextLocationLinks`.