import unittest
from quid.core.Quid import Quid
from test.Helper import get_text


class QuidTestCase(unittest.TestCase):

    def test_max_merge_ellipse_distance(self):
        source_text = 'This is a long Text but there are too many words here are even more words here to test.'
        target_text = 'This is a long Text ... even more words here to test.'

        quid = Quid(5, look_back_limit=2, max_merge_ellipse_distance=3)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(19, matches[0].source_span.end)
        self.assertEqual(0, matches[0].target_span.start)
        self.assertEqual(19, matches[0].target_span.end)

        self.assertEqual(58, matches[1].source_span.start)
        self.assertEqual(87, matches[1].source_span.end)
        self.assertEqual(24, matches[1].target_span.start)
        self.assertEqual(53, matches[1].target_span.end)

        self.assertEqual('This is a long Text',
                         get_text(source_text, matches[0].source_span))
        self.assertEqual('This is a long Text',
                         get_text(target_text, matches[0].target_span))

        quid = Quid(5, look_back_limit=2, max_merge_ellipse_distance=10)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(87, matches[0].source_span.end)
        self.assertEqual(0, matches[0].target_span.start)
        self.assertEqual(53, matches[0].target_span.end)

        self.assertEqual('This is a long Text but there are too many words here are even more words here to test.',
                         get_text(source_text, matches[0].source_span))
        self.assertEqual('This is a long Text ... even more words here to test.',
                         get_text(target_text, matches[0].target_span))

    def test_max_merge_distance(self):
        source_text = 'This is a long Text okay why do even more words here to test.'
        target_text = 'This is a long Text some not fitting even more words here to test.'

        quid = Quid(5, look_back_limit=2, max_merge_distance=2)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(19, matches[0].source_span.end)
        self.assertEqual(0, matches[0].target_span.start)
        self.assertEqual(19, matches[0].target_span.end)

        self.assertEqual(32, matches[1].source_span.start)
        self.assertEqual(61, matches[1].source_span.end)
        self.assertEqual(37, matches[1].target_span.start)
        self.assertEqual(66, matches[1].target_span.end)

        self.assertEqual('This is a long Text',
                         get_text(source_text, matches[0].source_span))
        self.assertEqual('This is a long Text',
                         get_text(target_text, matches[0].target_span))

        self.assertEqual('even more words here to test.',
                         get_text(source_text, matches[1].source_span))
        self.assertEqual('even more words here to test.',
                         get_text(target_text, matches[1].target_span))

        quid = Quid(5, look_back_limit=2, max_merge_distance=10)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(61, matches[0].source_span.end)
        self.assertEqual(0, matches[0].target_span.start)
        self.assertEqual(66, matches[0].target_span.end)

        self.assertEqual('This is a long Text okay why do even more words here to test.',
                         get_text(source_text, matches[0].source_span))
        self.assertEqual('This is a long Text some not fitting even more words here to test.',
                         get_text(target_text, matches[0].target_span))


if __name__ == '__main__':
    unittest.main()
