import unittest

from quid.passager.Passager import Passager
from quid.passager.TextWithMatches import TextWithMatches
from quid.match.Match import Match
from quid.match.MatchSpan import MatchSpan


class PassagerTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.passager = Passager()

    def test_passager(self):
        source_content = 'aa bb cc dd ee ff gg\nhh. dd ee ff gg hh ii.\njjj kkk lll mmm nnn ooo ppp\n' \
                         'qqq rrr sss ttt uuu vvv www\nxxx yyy zzz'
        text_1 = 'aa bb cc dd kkk lll mmm ...\nwww xxx yyy zzz. axc bzu fyo\nloi uiq dd ee ff gg hh\n' \
                 'ii. jjj kkk\ndd ee ff'

        match_1_source_match_segment = MatchSpan(50, 113)
        match_1_target_match_segment = MatchSpan(12, 44)
        match_1 = Match(match_1_source_match_segment, match_1_target_match_segment)

        match_2_source_match_segment = MatchSpan(26, 44)
        match_2_target_match_segment = MatchSpan(65, 83)
        match_2 = Match(match_2_source_match_segment, match_2_target_match_segment)

        matches = [match_1, match_2]

        target_text_matches = TextWithMatches('text 1', text_1, matches)
        target_text_matches_list = [target_text_matches]
        result = self.passager.generate(target_text_matches_list, source_content)

        self.assertIsNotNone(result)
        self.assertEqual(2, len(result.citation_source_links))
        self.assertEqual(2, len(result.citation_sources))
        self.assertEqual(2, len(result.target_text_location_links))
        self.assertEqual(1, len(result.target_texts))
        self.assertEqual(2, len(result.target_texts[0].target_locations))

    def test_conflicting_sources(self):
        source_content = 'Dies ist ein kurzer kleiner Test um das Erkennen von Overlap ein bisschen zu testen und außerdem sicherzustellen, dass auch vielfach Überlappungen erkannt werden müssen.'
        text_1 = 'Wir müssen, ist ein kurzer kleiner Test, und noch ein bisschen Text einbetten, Overlap ein bisschen zu testen, aber noch ein bisschen Text einbetten, auch vielfach Überlappungen erkannt werden.'
        text_2 = 'Dies ist ein kurzer kleiner Test um das Erkennen von Overlap ein bisschen zu testen und außerdem sicherzustellen, dass auch vielfach Überlappungen erkannt werden müssen.'

        # Text 1
        text_1_match_1_source_match_segment = MatchSpan(5, 32)
        text_1_match_1_target_match_segment = MatchSpan(12, 39)
        text_1_match_1 = Match(text_1_match_1_source_match_segment, text_1_match_1_target_match_segment)

        text_1_match_2_source_match_segment = MatchSpan(53, 83)
        text_1_match_2_target_match_segment = MatchSpan(79, 109)
        text_1_match_2 = Match(text_1_match_2_source_match_segment, text_1_match_2_target_match_segment)

        text_1_match_3_source_match_segment = MatchSpan(119, 161)
        text_1_match_3_target_match_segment = MatchSpan(150, 193)
        text_1_match_3 = Match(text_1_match_3_source_match_segment, text_1_match_3_target_match_segment)

        text_1_matches = [text_1_match_1, text_1_match_2, text_1_match_3]
        text_1_text_with_matches = TextWithMatches('text 1', text_1, text_1_matches)

        # Text 2
        text_2_match_1_source_match_segment = MatchSpan(0, 169)
        text_2_match_1_target_match_segment = MatchSpan(0, 169)
        text_2_match_1 = Match(text_2_match_1_source_match_segment, text_2_match_1_target_match_segment)

        text_2_matches = [text_2_match_1]
        text_2_text_with_matches = TextWithMatches('text 2', text_2, text_2_matches)

        texts_with_matches = [text_1_text_with_matches, text_2_text_with_matches]
        result = self.passager.generate(texts_with_matches, source_content)

        self.assertIsNotNone(result)
        self.assertEqual(1, len(result.citation_sources))

        rsc = result.citation_sources[0]

        self.assertEqual(7, len(rsc.source_segments))
        self.assertEqual(1, rsc.source_segments[0].frequency)
        self.assertEqual(2, rsc.source_segments[1].frequency)
        self.assertEqual(1, rsc.source_segments[2].frequency)
        self.assertEqual(2, rsc.source_segments[3].frequency)
        self.assertEqual(1, rsc.source_segments[4].frequency)
        self.assertEqual(2, rsc.source_segments[5].frequency)
        self.assertEqual(1, rsc.source_segments[6].frequency)

    # def test_important_segments(self):
    #     source_content = 'Dies ist ein kurzer kleiner Test um die Erstellung von important segments ein bisschen zu testen und außerdem.'
    #     text_1 = 'Dies ist ein kurzer kleiner Test um die Erstellung'
    #     text_2 = 'um die Erstellung von important segments ein bisschen zu testen und außerdem.'
    #     text_3 = 'um die Erstellung von important segments ein bisschen zu testen und außerdem.'
    #
    #     # Text 1
    #     text_1_match_1_source_match_segment = MatchSpan(0, 50)
    #     text_1_match_1_target_match_segment = MatchSpan(0, 50)
    #     text_1_match_1 = Match(text_1_match_1_source_match_segment, text_1_match_1_target_match_segment)
    #
    #     text_1_matches = [text_1_match_1]
    #     text_1_text_with_matches = TextWithMatches('text 1', text_1, text_1_matches)
    #
    #     # Text 2
    #     text_2_match_1_source_match_segment = MatchSpan(33, 110)
    #     text_2_match_1_target_match_segment = MatchSpan(0, 77)
    #     text_2_match_1 = Match(text_2_match_1_source_match_segment, text_2_match_1_target_match_segment)
    #
    #     text_2_matches = [text_2_match_1]
    #     text_2_text_with_matches = TextWithMatches('text 2', text_2, text_2_matches)
    #
    #     # Text 3
    #     text_3_match_1_source_match_segment = MatchSpan(33, 110)
    #     text_3_match_1_target_match_segment = MatchSpan(0, 77)
    #     text_3_match_1 = Match(text_3_match_1_source_match_segment, text_3_match_1_target_match_segment)
    #
    #     text_3_matches = [text_3_match_1]
    #     text_3_text_with_matches = TextWithMatches('text 3', text_3, text_3_matches)
    #
    #     texts_with_matches = [text_1_text_with_matches, text_2_text_with_matches, text_3_text_with_matches]
    #     result = self.passager.generate(texts_with_matches, source_content)
    #
    #     self.assertIsNotNone(result)
    #     self.assertEqual(1, len(result.citation_sources))
    #
    #     rsc = result.citation_sources[0]
    #
    #     self.assertEqual(3, len(rsc.source_segments))
    #     self.assertEqual(2, len(rsc.important_segments))
    #
    # def test_important_segments_2(self):
    #     source_content = 'Dies ist ein kurzer kleiner Test um die Erstellung von important segments ein bisschen zu testen und außerdem.'
    #     text_1 = 'Dies ist ein kurzer kleiner Test um die Erstellung von important segments'
    #     text_2 = 'Dies ist ein kurzer kleiner Test um die Erstellung von important segments'
    #     text_3 = 'um die Erstellung von important segments ein bisschen zu testen und außerdem.'
    #     text_4 = 'um die Erstellung von important segments'
    #     text_5 = 'um die Erstellung von important segments'
    #
    #     # Text 1
    #     text_1_match_1_source_match_segment = MatchSpan(0, 73)
    #     text_1_match_1_target_match_segment = MatchSpan(0, 73)
    #     text_1_match_1 = Match(text_1_match_1_source_match_segment, text_1_match_1_target_match_segment)
    #
    #     text_1_matches = [text_1_match_1]
    #     text_1_text_with_matches = TextWithMatches('text 1', text_1, text_1_matches)
    #
    #     # Text 2
    #     text_2_match_1_source_match_segment = MatchSpan(0, 73)
    #     text_2_match_1_target_match_segment = MatchSpan(0, 73)
    #     text_2_match_1 = Match(text_2_match_1_source_match_segment, text_2_match_1_target_match_segment)
    #
    #     text_2_matches = [text_2_match_1]
    #     text_2_text_with_matches = TextWithMatches('text 2', text_2, text_2_matches)
    #
    #     # Text 3
    #     text_3_match_1_source_match_segment = MatchSpan(33, 110)
    #     text_3_match_1_target_match_segment = MatchSpan(0, 77)
    #     text_3_match_1 = Match(text_3_match_1_source_match_segment, text_3_match_1_target_match_segment)
    #
    #     text_3_matches = [text_3_match_1]
    #     text_3_text_with_matches = TextWithMatches('text 3', text_3, text_3_matches)
    #
    #     # Text 4
    #     text_4_match_1_source_match_segment = MatchSpan(33, 73)
    #     text_4_match_1_target_match_segment = MatchSpan(0, 40)
    #     text_4_match_1 = Match(text_4_match_1_source_match_segment, text_4_match_1_target_match_segment)
    #
    #     text_4_matches = [text_4_match_1]
    #     text_4_text_with_matches = TextWithMatches('text 4', text_4, text_4_matches)
    #
    #     # Text 5
    #     text_5_match_1_source_match_segment = MatchSpan(33, 73)
    #     text_5_match_1_target_match_segment = MatchSpan(0, 40)
    #     text_5_match_1 = Match(text_5_match_1_source_match_segment, text_5_match_1_target_match_segment)
    #
    #     text_5_matches = [text_5_match_1]
    #     text_5_text_with_matches = TextWithMatches('text 5', text_5, text_5_matches)
    #
    #     texts_with_matches = [text_1_text_with_matches, text_2_text_with_matches, text_3_text_with_matches,
    #                           text_4_text_with_matches, text_5_text_with_matches]
    #     result = self.passager.generate(texts_with_matches, source_content)
    #
    #     self.assertIsNotNone(result)
    #     self.assertEqual(1, len(result.citation_sources))
    #
    #     rsc = result.citation_sources[0]
    #
    #     self.assertEqual(3, len(rsc.source_segments))
    #     # self.assertEqual(3, len(rsc.important_segments))
