import unittest

from quid.core.Quid import Quid


class SetupTestCase(unittest.TestCase):

    def test_setup_working(self):
        try:
            Quid(5)
        except ValueError:
            self.fail("ValueError raised unexpectedly!")

    def test_setup_incorrect_min_match_length(self):
        with self.assertRaises(Exception) as context:
            Quid(0)
        self.assertEqual('min match length must be >= 1', context.exception.args[0])

    def test_setup_incorrect_look_back_limit(self):
        with self.assertRaises(Exception) as context:
            Quid(3, look_back_limit=-1)
        self.assertEqual('look back limit must be positive', context.exception.args[0])

    def test_setup_incorrect_look_ahead_limit(self):
        with self.assertRaises(Exception) as context:
            Quid(3, look_ahead_limit=-1)
        self.assertEqual('look ahead limit must be positive', context.exception.args[0])

    def test_setup_incorrect_max_merge_distance(self):
        with self.assertRaises(Exception) as context:
            Quid(3, max_merge_distance=-1)
        self.assertEqual('max merge distance must be positive', context.exception.args[0])

    def test_setup_incorrect_max_merge_ellipse_distance(self):
        with self.assertRaises(Exception) as context:
            Quid(3, max_merge_ellipse_distance=-1)
        self.assertEqual('max merge ellipse distance must be positive', context.exception.args[0])

    def test_setup_min_levenshtein_distance_too_small(self):
        with self.assertRaises(Exception) as context:
            Quid(3, min_levenshtein_similarity=-1.0)
        self.assertEqual('min levenshtein similarity must be between 0 and 1', context.exception.args[0])

    def test_setup_min_levenshtein_distance_too_big(self):
        with self.assertRaises(Exception) as context:
            Quid(3, min_levenshtein_similarity=5.0)
        self.assertEqual('min levenshtein similarity must be between 0 and 1', context.exception.args[0])

    def test_empty_source_input(self):
        quid = Quid(5)
        matches = quid.compare('', 'test')

        self.assertEqual(0, len(matches))

    def test_empty_target_input(self):
        quid = Quid(5)
        matches = quid.compare('test', '')

        self.assertEqual(0, len(matches))


if __name__ == '__main__':
    unittest.main()
