import unittest
from quid.core.Quid import Quid


class QuidTestCase(unittest.TestCase):

    def test_splitting(self):
        source_text = 'This is a long Text but there are too many words here are even more words here to test.'
        target_text = 'This is a long Text but there are too many words here are even more words here to test.'

        quid = Quid(min_match_length=2, split_long_texts=True, split_length=5, max_num_processes=2)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(87, matches[0].source_span.end)

    def test_splitting_border_mid_1(self):
        source_text = 'This is a long Text but just some random words'
        target_text = 'This is a long Text but house river cat dog'

        quid = Quid(min_match_length=2, split_long_texts=True, split_length=5, max_num_processes=2)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(23, matches[0].source_span.end)

    def test_splitting_border_mid_2(self):
        source_text = 'This is a long Text but just some and now a quote again'
        target_text = 'This is a long Text but house river and now a quote again'

        quid = Quid(min_match_length=2, max_merge_distance=1, split_long_texts=True, split_length=5,
                    max_num_processes=2)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(23, matches[0].source_span.end)
        self.assertEqual(34, matches[1].source_span.start)
        self.assertEqual(55, matches[1].source_span.end)

    def test_splitting_border_end(self):
        source_text = 'This is a long Text but'
        target_text = 'This is a long Text but'

        quid = Quid(min_match_length=2, split_long_texts=True, split_length=5, max_num_processes=2)
        matches = quid.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_span.start)
        self.assertEqual(23, matches[0].source_span.end)
