import unittest

from quid.passager.CitationSource import CitationSource
from quid.passager.CitationSourceLink import CitationSourceLink
from quid.passager.SourceSegment import SourceSegment
from quid.passager.TargetLocation import TargetLocation
from quid.passager.TargetLocationSelection import TargetLocationSelection
from quid.passager.TargetText import TargetText
from quid.visualization.TargetTextWithContent import TargetTextWithContent
from quid.visualization.Visualizer import Visualizer


class VisualizerTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.visualizer = Visualizer(False, 25)

    def test_visualize(self):
        source_content = 'aa bb cc dd ee ff gg\nhh. dd ee ff gg hh ii.\njjj kkk lll mmm nnn ooo ppp\n' \
                         'qqq rrr sss ttt uuu vvv www\nxxx yyy zzz'

        source_segment_1 = SourceSegment(3, 26, 44, 1, 6, 'd ee ff gg hh ii.')
        # important_segment_1 = ImportantSegment(5, [3], 1.0, 6, 'd ee ff gg hh ii.')

        source_segment_2 = SourceSegment(1, 50, 113, 1, 14, 'k lll mmm nnn ooo ppp\nqqq rrr sss ttt uuu vvv www\n'
                                                            'xxx yyy zzz')
        # important_segment_2 = ImportantSegment(6, [1], 1.0, 14, 'k lll mmm nnn ooo ppp\nqqq rrr sss ttt uuu vvv www\n'
        #                                                         'xxx yyy zzz')

        citation_source_1 = CitationSource(4, [source_segment_1], 'd ee ff gg hh ii.')
        citation_source_2 = CitationSource(2, [source_segment_2], 'k lll mmm nnn ooo ppp\n'
                                                                  'qqq rrr sss ttt uuu vvv www\n'
                                                                  'xxx yyy zzz')

        citation_sources = [citation_source_1, citation_source_2]

        target_location_selection_1 = TargetLocationSelection.from_value(7, 9)
        citation_source_link_1 = CitationSourceLink(4, [target_location_selection_1])

        target_location_selection_2 = TargetLocationSelection.from_value(7, 8)
        citation_source_link_2 = CitationSourceLink(2, [target_location_selection_2])

        citation_source_links = [citation_source_link_1, citation_source_link_2]

        target_location_1 = TargetLocation(8, 12, 44, 'kkk lll mmm ...\nwww xxx yyy zzz.')
        target_location_2 = TargetLocation(9, 65, 83, 'dd ee ff gg hh\nii.')

        target_text = TargetText(7, 'text 1', [target_location_1, target_location_2])

        target_text_content = 'aa bb cc dd kkk lll mmm ...\nwww xxx yyy zzz. axc bzu fyo\nloi uiq dd ee ff gg hh\n' \
                              'ii. jjj kkk\ndd ee ff'

        target_texts_with_content = [TargetTextWithContent(target_text, target_text_content)]

        visualization = self.visualizer.visualize('Test', 'Tester', 2021, source_content, citation_sources,
                                                  citation_source_links, target_texts_with_content)

        self.assertEqual('Test', visualization.info.title)
        self.assertEqual('Tester', visualization.info.author)
        self.assertEqual(1, len(visualization.targets_html))
